@@ -8,19 +8,21 @@ filter 'DBI::db', sub {
     my ($dbh, $ddp) = @_;
     my $name = $dbh->{Driver}{Name};
 
-    my $string = "$name Database Handle ("
+    my $string = "$name Database Handle "
+               . $ddp->maybe_colorize('(', 'brackets')
                . _get_db_status($dbh->{Active}, $ddp)
-               . ')'
+               . $ddp->maybe_colorize(')', 'brackets')
                ;
     return $string
         if exists $ddp->extra_config->{filter_db}{connection_details}
            && !$ddp->extra_config->{filter_db}{connection_details};
 
-    $string .= ' {';
+    $string .= ' ' . $ddp->maybe_colorize('{', 'brackets');
     $ddp->indent;
     my %dsn = split( /[;=]/, $dbh->{Name} );
     foreach my $k (keys %dsn) {
-        $string .= $ddp->newline . "$k: " . $dsn{$k};
+        $string .= $ddp->newline . $k . $ddp->maybe_colorize(':', 'separator')
+                . ' ' . $dsn{$k};
     }
     $string .= $ddp->newline . 'Auto Commit: ' . $dbh->{AutoCommit};
 
@@ -37,7 +39,7 @@ filter 'DBI::db', sub {
             . $ddp->maybe_colorize(($dbh->{Statement} || '-'), 'string');
 
     $ddp->outdent;
-    $string .= $ddp->newline . '}';
+    $string .= $ddp->newline . $ddp->maybe_colorize('}', 'brackets');
     return $string;
 };
 
@@ -80,11 +82,16 @@ filter 'DBIx::Class::Schema' => sub {
         : $ddp->class->expand
         ;
     my $connected = _get_db_status($storage->connected, $ddp);
-    return "$name (" . $storage->sqlt_type . " - $connected)"
-        unless $expand;
+    if (!$expand) {
+        return "$name " . $ddp->maybe_colorize('(', 'brackets')
+            . $storage->sqlt_type . " - $connected"
+            . $ddp->maybe_colorize(')', 'brackets')
+            ;
+    }
 
     $ddp->indent;
-    my $output = $name . ' {' . $ddp->newline
+    my $output = $name . ' ' . $ddp->maybe_colorize('{', 'brackets')
+        . $ddp->newline
         . 'connection: ' . ($config->{show_handle}
             ? $ddp->parse($storage->dbh)
             : $storage->sqlt_type . " Database Handle ($connected)"
@@ -95,12 +102,12 @@ filter 'DBIx::Class::Schema' => sub {
     my $load_sources = 'names';
     if (exists $config->{loaded_sources}) {
         my $type = $config->{loaded_sources};
-        if ($type && ($type eq 'names' || $type eq 'expand' || $type eq 'none')) {
+        if ($type && ($type eq 'names' || $type eq 'details' || $type eq 'none')) {
             $load_sources = $type;
         }
         else {
             Data::Printer::Common::_warn(
-                "filter_db.schema.loaded_sources must be names,expand or none"
+                "filter_db.schema.loaded_sources must be names, details or none"
             );
         }
     }
@@ -121,13 +128,13 @@ filter 'DBIx::Class::Schema' => sub {
             foreach my $i (0 .. $#sources) {
                 my $source = $schema->source($sources[$i]);
                 $output .= $ddp->newline . $ddp->parse($source);
-                $output .= ',' if $i < $#sources;
+                $output .= $ddp->maybe_colorize(',', 'separator') if $i < $#sources;
             }
             $ddp->outdent;
         }
     }
     $ddp->outdent;
-    $output .= $ddp->newline . '}';
+    $output .= $ddp->newline . $ddp->maybe_colorize('}', 'brackets');
     return $output;
 };
 
@@ -135,7 +142,9 @@ filter 'DBIx::Class::Row' => sub {
     my ($row, $ddp) = @_;
 
     my $output = $row->result_source->source_name
-               . ' Row (' . ($row->in_storage ? '' : 'NOT ') . 'in storage) {';
+               . ' Row ' . $ddp->maybe_colorize('(', 'brackets')
+               . ($row->in_storage ? '' : 'NOT ') . 'in storage'
+               . $ddp->maybe_colorize(') {', 'brackets');
 
     $ddp->indent;
     my %orig_columns = map { $_ => 1 } $row->columns;
@@ -148,17 +157,30 @@ filter 'DBIx::Class::Row' => sub {
         my $l = length $col;
         $longest = $l if $l > $longest;
     }
+    my $show_updated_label = !exists $ddp->extra_config->{filter_db}{show_updated_label}
+                          || $ddp->extra_config->{filter_db}{show_updated_label};
+    my $show_extra_label = !exists $ddp->extra_config->{filter_db}{show_extra_label}
+                        || $ddp->extra_config->{filter_db}{show_extra_label};
+
     foreach my $col (@ordered) {
         my $padding = $longest - length($col);
-        $output .= $ddp->newline . $col . ': ' . (' ' x $padding)
-            . $data{$col}
-            . (' (updated)')x!!(exists $dirty{$col}) # show_updated_label, colorize_updated
-            . (' (extra)')x!!(!exists $orig_columns{$col}) # show_extra_label, colorize_extra
-            ;
+        my $content = $data{$col};
+        $output .= $ddp->newline . $col
+                . $ddp->maybe_colorize(':', 'separator')
+                . ' ' . (' ' x $padding)
+                . $ddp->parse(\$content, seen_override => 1)
+                ;
+
+        if (exists $dirty{$col} && $show_updated_label) {
+            $output .= ' (updated)';
+        }
+        if (!exists $orig_columns{$col} && $show_extra_label) {
+            $output .= ' (extra)';
+        }
     }
     # TODO: methods: foo, bar <-- follows class.*, but can be overriden by filter_db.class.*
     $ddp->outdent;
-    $output .= $ddp->newline . '}';
+    $output .= $ddp->newline . $ddp->maybe_colorize('}', 'brackets');
     return $output;
 };
 
@@ -167,7 +189,8 @@ filter 'DBIx::Class::ResultSet' => sub {
 
     $ddp->indent;
     my $output = $rs->result_source->source_name
-               . ' ResultSet {' . $ddp->newline;
+               . ' ResultSet ' . $ddp->maybe_colorize('{', 'brackets')
+               . $ddp->newline;
 
     # NOTE: we're totally breaking DBIC's encapsulation here. But since DDP
     # is a tool to inspect the inner workings of objects, it's okay. Ish.
@@ -185,7 +208,7 @@ filter 'DBIx::Class::ResultSet' => sub {
         }
     }
     else {
-        $output .= '(unable to lookup - patches welcome!)';
+        $output .= $ddp->maybe_colorize('(unable to lookup - patches welcome!)', 'unknown');
     }
     # TODO: show joins/prefetches/from
     # TODO: look at get_cache() for results
@@ -200,7 +223,8 @@ filter 'DBIx::Class::ResultSet' => sub {
                 ;
         if (@query_data) {
             $output .= $ddp->newline . join( $ddp->newline, map {
-                    $_->[1] . ' (' . $_->[0]{sqlt_datatype} . ')'
+                    $_->[1] . ' ' . $ddp->maybe_colorize('(', 'brackets')
+                    . $_->[0]{sqlt_datatype} . $ddp->maybe_colorize(')', 'brackets')
                   } @query_data
                 );
         }
@@ -214,7 +238,7 @@ filter 'DBIx::Class::ResultSet' => sub {
     }
 
     $ddp->outdent;
-    $output .= $ddp->newline . '}';
+    $output .= $ddp->newline . $ddp->maybe_colorize('}', 'brackets');
     return $output;
 };
 
@@ -224,25 +248,90 @@ filter 'DBIx::Class::ResultSource' => sub {
 
     my $output = $source->source_name . ' ResultSource';
     if ($source->isa('DBIx::Class::ResultSource::View')) {
-        $output .= ' ('
+        $output .= ' ' . $ddp->maybe_colorize('(', 'brackets')
             . ($source->is_virtual ? 'Virtual ' : '')
-            . 'View)'
+            . 'View' . $ddp->maybe_colorize(')', 'brackets')
             ;
     }
-    $ddp->indent;
-    # TODO: filter_db.show_source_table
-    # TODO: filter_db.describe_columns
-    $output .= ' {' . $ddp->newline
-            . 'table: ' . $ddp->parse(\$source->name)
-            ;
 
-    my $columns = $source->columns_info;
+    my $show_source_table = !exists $ddp->extra_config->{filter_db}{show_source_table}
+                         || $ddp->extra_config->{filter_db}{show_source_table};
+    my $column_info = 'details';
+    if (exists $ddp->extra_config->{filter_db}{column_info}) {
+        my $new = $ddp->extra_config->{filter_db}{column_info};
+        if ($new && ($new eq 'names' || $new eq 'details' || $new eq 'none')) {
+            $column_info = $new;
+        }
+        else {
+            Data::Printer::Common::_warn(
+                "filter_db.column_info must be names, details or none"
+            );
+        }
+    }
+    return $output if !$show_source_table && $column_info eq 'none';
+
+    $ddp->indent;
+    $output .= ' ' . $ddp->maybe_colorize('{', 'brackets');
+    if ($show_source_table) {
+        $output .= $ddp->newline . 'table: ' . $ddp->parse(\$source->name);
+    }
+    if ($column_info ne 'none') {
+        my $columns = $source->columns_info;
+        $output .= $ddp->newline . 'columns:';
+        $output .= ' - ' unless %$columns;
+        my $separator = $ddp->maybe_colorize(',', 'separator') . ' ';
+        if ($column_info eq 'names') {
+            my %parsed_cols = map { $_ => 1 } keys %$columns;
+            my @primary = Data::Printer::Common::_nsort($source->primary_columns);
+            if (@primary) {
+                delete $parsed_cols{$_} foreach @primary;
+                $output .= ' ' . join($separator => map {
+                        $ddp->maybe_colorize($_, 'method') . ' (primary)'
+                    } @primary
+                );
+                $output .= ',' if keys %parsed_cols;
+            }
+            if (keys %parsed_cols) {
+                $output .= ' ' . join($separator => map {
+                        $ddp->maybe_colorize($_, 'method')
+                    } Data::Printer::Common::_nsort(keys %parsed_cols)
+                );
+            }
+        }
+        else { # details!
+            $output .= _show_column_details($source, $columns, $ddp);
+        }
+        my %uniques = $source->unique_constraints;
+        delete $uniques{primary};
+        if (keys %uniques) {
+            $output .= $ddp->newline . 'non-primary uniques:';
+            $ddp->indent;
+            foreach my $key (Data::Printer::Common::_nsort(keys %uniques)) {
+                $output .= $ddp->newline
+                        . $ddp->maybe_colorize('(', 'brackets')
+                        . join($separator, @{$uniques{$key}})
+                        . $ddp->maybe_colorize(')', 'brackets') . " as '$key'"
+                        ;
+            }
+            $ddp->outdent;
+        }
+
+        # TODO: use $source->relationships and $source->relationship_info
+        # to list relationships between sources. (filter_db.show_relationships
+        # TODO: public methods implemented by the user
+        # TODO; "current result count" (touching the db)
+        # TODO: "first X eresults" (touching the db)
+    }
+    $ddp->outdent;
+    return $output . $ddp->newline . $ddp->maybe_colorize('}', 'brackets');
+};
+
+sub _show_column_details {
+    my ($source, $columns, $ddp) = @_;
+    my $output = '';
     my %parsed_columns;
-    my $has_meta;
     foreach my $colname (keys %$columns) {
         my $meta = $columns->{$colname};
-        next unless keys %$meta;
-        $has_meta = 1;
         my $parsed = ' ';
         if (exists $meta->{data_type} && defined $meta->{data_type}) {
             $parsed .= $meta->{data_type};
@@ -271,7 +360,7 @@ filter 'DBIx::Class::ResultSource' => sub {
             }
         }
         else {
-            $parsed .= '(unknown data type)';
+            $parsed .= $ddp->maybe_colorize('(unknown data type)', 'unknown');
         }
         if (exists $meta->{is_nullable}) {
             $parsed .= ((' not')x !$meta->{is_nullable}) . ' null';
@@ -300,66 +389,33 @@ filter 'DBIx::Class::ResultSource' => sub {
 
     my @primary_keys = $source->primary_columns;
     if (keys %parsed_columns || @primary_keys) {
-        $output .= $ddp->newline . 'columns:';
-        if ($has_meta) {
-            $ddp->indent;
-            foreach my $colname (@primary_keys) {
-                my $value = exists $parsed_columns{$colname}
-                    ? delete $parsed_columns{$colname} : '';
-                $output .= $ddp->newline . $colname
-                        . (defined $value ? $value : '')
-                        . ' (primary)'
-                        . (keys %parsed_columns ? ',' : '')
-                        ;
-            }
-            if (keys %parsed_columns) {
-                my @sorted_columns = Data::Printer::Common::_nsort(keys %parsed_columns);
-                foreach my $i (0 .. $#sorted_columns) {
-                    my $colname = $sorted_columns[$i];
-                    # TODO: v-align column names (like hash keys)
-                    $output .= $ddp->newline . $colname
-                    . $parsed_columns{$colname}
-                    . ($i == $#sorted_columns ? '' : ',')
+        my $separator = $ddp->maybe_colorize(',', 'separator');
+        $ddp->indent;
+        foreach my $colname (@primary_keys) {
+            my $value = exists $parsed_columns{$colname}
+                ? delete $parsed_columns{$colname} : '';
+            $output .= $ddp->newline . $colname
+                    . (defined $value ? $value : '')
+                    . ' (primary)'
+                    . (keys %parsed_columns ? $separator : '')
                     ;
-                }
-            }
-            $ddp->outdent;
         }
-        else {
-            foreach my $colname (@primary_keys) {
-                # TODO: filter_db.show_primary_keys
-                $output .= $colname . ' (primary)';
-                delete $parsed_columns{$colname};
-                $output .= ', 'if keys %parsed_columns;
-            }
-            if (keys %parsed_columns) {
-                my @sorted_cols = Data::Printer::Common::_nsort(keys %parsed_columns);
-                $output .= join(', ' => @sorted_cols);
+        if (keys %parsed_columns) {
+            my @sorted_columns = Data::Printer::Common::_nsort(keys %parsed_columns);
+            foreach my $i (0 .. $#sorted_columns) {
+                my $colname = $sorted_columns[$i];
+                # TODO: v-align column names (like hash keys)
+                $output .= $ddp->newline . $colname
+                . $parsed_columns{$colname}
+                . ($i == $#sorted_columns ? '' : $separator)
+                ;
             }
         }
-        my %uniques = $source->unique_constraints;
-        delete $uniques{primary};
-        if (keys %uniques) {
-            $output .= $ddp->newline . 'non-primary uniques:';
-            $ddp->indent;
-            foreach my $key (Data::Printer::Common::_nsort(keys %uniques)) {
-                $output .= $ddp->newline
-                        . '(' . join(',', @{$uniques{$key}})
-                        . ") as '$key'"
-                        ;
-            }
-            $ddp->outdent;
-        }
-
-        # TODO: use $source->relationships and $source->relationship_info
-        # to list relationships between sources. (filter_db.show_relationships
-        # TODO: public methods implemented by the user
-        # TODO; "current result count" (touching the db)
-        # TODO: "first X eresults" (touching the db)
+        $ddp->outdent;
     }
-    $ddp->outdent;
-    return $output . $ddp->newline . '}';
-};
+    return $output;
+}
+
 
 sub _get_db_status {
     my ($status, $ddp) = @_;
@@ -369,39 +425,6 @@ sub _get_db_status {
         ;
 }
 
-filter '-class' => sub {
-    my ($obj, $ddp) = @_;
-
-    # TODO: if it's a Result, show columns and relationships (anything that
-    #       doesn't involve touching the database
-    if ( grep { $obj->isa($_) } qw(DBIx::Class::ResultSet DBIx::Class::ResultSetColumn) ) {
-
-        my $str = $ddp->maybe_colorize( ref($obj), 'class' );
-        $str .= ' (' . $obj->result_class . ')'
-          if $obj->can( 'result_class' );
-
-        if (my $query_data = $obj->as_query) {
-          my @query_data = @$$query_data;
-          $ddp->indent;
-          my $sql = shift @query_data;
-          $str .= ' {'
-               . $ddp->newline . $ddp->maybe_colorize($sql, 'string')
-               . $ddp->newline . join ( $ddp->newline, map {
-                      $_->[1] . ' (' . $_->[0]{sqlt_datatype} . ')'
-                    } @query_data
-               )
-               ;
-          $ddp->outdent;
-          $str .= $ddp->newline . '}';
-        }
-
-        return $str;
-    }
-    else {
-        return;
-    }
-};
-
 1;
 __END__
 
@@ -415,13 +438,43 @@ In your C<.dataprinter> file:
 
     filters = DB
 
-You may also customize the look and feel with the following options:
+You may also customize the look and feel with the following options
+(defaults shown):
 
-    filter_db.expand_dbh = 1
+    ### DBH settings:
 
-    # you can even customize your themes:
-    colors.filter_db_connected    = #00cc00
-    colors.filter_db_disconnected = #cc0000
+    # expand database handle objects
+    filter_db.connection_details = 1
+
+
+    ### DBIx::Class settings:
+
+    # signal when a result column is dirty:
+    filter_db.show_updated_label = 1
+
+    # signal when result rows contain extra columns:
+    filter_db.show_extra_label = 1
+
+    # override class.expand for schema dump
+    filter_db.schema.expand = 1
+
+    # expand DBH handle on schema dump (may touch DB)
+    filter_db.schema.show_handle = 0
+
+    # show source details (connected tables) on schema dump
+    # (may be set to 'names', 'details' or 'none')
+    filter_db.schema.loaded_sources = names
+
+    # show source table name ResultSource objects
+    filter_db.show_source_table = 1
+
+    # show source columns ('names', 'details' or 'none'):
+    filter_db.column_info = details
+
+    # this plugin honors theme colors where applicable
+    # and provides the following custom colors for you to use:
+    colors.filter_db_connected    = #a0d332
+    colors.filter_db_disconnected = #b3422d
 
 That's it!
 
@@ -490,7 +543,7 @@ Other available options for the schema are (default values shown):
     # to fetch a healthy DBH:
     filter_db.schema.show_handle = 0
 
-    # set to 'expand' to view source details, or 'none' to skip it:
+    # set to 'details' to view source details, or 'none' to skip it:
     filter_db.schema.loaded_sources = names
 
 B<DBIx::Class::ResultSource> objects will be expanded to show details
